@@ -40,6 +40,7 @@ import gtk_toolbox
 
 
 _moduleLogger = logging.getLogger("dc_glade")
+PROFILE_STARTUP = False
 
 
 def getmtime_nothrow(path):
@@ -73,6 +74,7 @@ class Dialcentral(object):
 	ACCOUNT_TAB = 4
 
 	NULL_BACKEND = 0
+	# 1 Was GrandCentral support so the gap was maintained for compatibility
 	GV_BACKEND = 2
 	BACKENDS = (NULL_BACKEND, GV_BACKEND)
 
@@ -162,7 +164,7 @@ class Dialcentral(object):
 
 		self._window.connect("destroy", self._on_close)
 		self._window.set_default_size(800, 300)
-		self._window.show_all()
+		self._window.show()
 
 		self._loginSink = gtk_toolbox.threaded_stage(
 			gtk_toolbox.comap(
@@ -171,9 +173,12 @@ class Dialcentral(object):
 			)
 		)
 
-		backgroundSetup = threading.Thread(target=self._idle_setup)
-		backgroundSetup.setDaemon(True)
-		backgroundSetup.start()
+		if not PROFILE_STARTUP:
+			backgroundSetup = threading.Thread(target=self._idle_setup)
+			backgroundSetup.setDaemon(True)
+			backgroundSetup.start()
+		else:
+			self._idle_setup()
 
 	def _idle_setup(self):
 		"""
@@ -217,13 +222,17 @@ class Dialcentral(object):
 
 			try:
 				import alarm_handler
-				self._alarmHandler = alarm_handler.AlarmHandler()
+				if alarm_handler.AlarmHandler is not alarm_handler._NoneAlarmHandler:
+					self._alarmHandler = alarm_handler.AlarmHandler()
+				else:
+					self._alarmHandler = None
 			except (ImportError, OSError):
 				alarm_handler = None
 			except Exception:
 				with gtk_toolbox.gtk_lock():
 					self._errorDisplay.push_exception()
 				alarm_handler = None
+			if alarm_handler is None:
 				_moduleLogger.warning("No notification support")
 			if hildonize.IS_HILDON_SUPPORTED:
 				try:
@@ -357,7 +366,7 @@ class Dialcentral(object):
 
 			serviceId = self.NULL_BACKEND
 			loggedIn = False
-			if not force:
+			if not force and self._defaultBackendId != self.NULL_BACKEND:
 				with gtk_toolbox.gtk_lock():
 					banner = hildonize.show_busy_banner_start(self._window, "Logging In...")
 				try:
@@ -491,7 +500,10 @@ class Dialcentral(object):
 		@note UI Thread
 		"""
 		try:
-			self._defaultBackendId = config.getint(constants.__pretty_app_name__, "active")
+			if not PROFILE_STARTUP:
+				self._defaultBackendId = config.getint(constants.__pretty_app_name__, "active")
+			else:
+				self._defaultBackendId = self.NULL_BACKEND
 			blobs = (
 				config.get(constants.__pretty_app_name__, "bin_blob_%i" % i)
 				for i in xrange(len(self._credentials))
@@ -569,8 +581,15 @@ class Dialcentral(object):
 		"""
 		@note Thread Agnostic
 		"""
+		# Because we now only support GVoice, if there are user credentials,
+		# always assume its using the GVoice backend
+		if self._credentials[0] and self._credentials[1]:
+			backend = self.GV_BACKEND
+		else:
+			backend = self.NULL_BACKEND
+
 		config.add_section(constants.__pretty_app_name__)
-		config.set(constants.__pretty_app_name__, "active", str(self._selectedBackendId))
+		config.set(constants.__pretty_app_name__, "active", str(backend))
 		config.set(constants.__pretty_app_name__, "orientation", str(int(gtk_toolbox.get_screen_orientation())))
 		for i, value in enumerate(self._credentials):
 			blob = base64.b64encode(value)
@@ -861,7 +880,8 @@ def run_dialpad():
 	if hildonize.IS_HILDON_SUPPORTED:
 		gtk.set_application_name(constants.__pretty_app_name__)
 	handle = Dialcentral()
-	gtk.main()
+	if not PROFILE_STARTUP:
+		gtk.main()
 
 
 class DummyOptions(object):
